#!/usr/bin/python
# -*- coding: utf-8 -*-

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

from aero.apps.healthcheck.handlers import HealthcheckHandler

urls = (
    (r'/healthcheck/?', HealthcheckHandler),
)
